#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2020, Intel Corporation. All rights reserved.
# SPDX-License-Identifier: BSD-2-Clause
#


class NoDevicesFoundError(Exception):
    def __init__(self, device_type):
        self.device_type = device_type

    def __str__(self):
        return "No {} devices found. Please check your connection and ensure the device in recovery mode".format(self.device_type)


class DeviceModeError(Exception):
    def __init__(self, expected, device):
        self.expected = expected
        self.device = device

    def __str__(self):
        error_str = "Device in wrong mode. Expected mode is {} but device {} is in {} mode.".format(
            self.expected, self.device, self.device.status
        )
        if self.expected == "Fip recovery":
            hint_str = "You may need to power down, set the recovery switch and boot the device."
        elif self.expected == "OS recovery" and self.device.status == "Fip recovery":
            hint_str = "You may need to flash FIP first."
        elif self.expected == "OS recovery" and self.device.status == "Ready":
            hint_str = "You may need to power down, set the recovery switch, boot the device and flash FIP first."
        else:
            hint_str = ""
        return "{} {}".format(error_str, hint_str)
