#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (c) 2020, Intel Corporation. All rights reserved.
# SPDX-License-Identifier: BSD-2-Clause
#

import os
import shutil
import time
import platform
import sys
import subprocess
import logging
from click import BadParameter
from ctypes import c_uint32

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

class XlinkWrapperWinManager():
    def __init__(self, slot=None):
        self.logger = logger
        self.slot = slot
        self.xlink_path = os.path.join(os.path.dirname(sys.path[0]),'site-packages','flashtool', 'thirdparty', 'xlink')

 
    def recover(self, fip, **kwargs):
        max_threads = kwargs.get("max_threads", False)
        if not os.path.exists("C:\\intel-kmb\\"):
            os.mkdir("C:\\intel-kmb\\")
        shutil.copy(fip, "C:\\intel-kmb\\fip.bin")
        self.logger.debug("Starting FIP flashing")
        # Run xlinkFlashFip.exe
        flashFipArgs = [os.path.join(self.xlink_path,'XLinkFlashFip.exe')]
        if self.slot is not None:
            flashFipArgs.append("slot")
            flashFipArgs.append(self.slot)
        process = subprocess.Popen(flashFipArgs)
        process.wait()
        if process.returncode == 0:
            self.logger.info("FIP flashed successfully")
            if os.path.basename(fip) == "fip-m2.bin":
                _ = input("Disable \"Intel(R) KeemBay XLink Core Device\" then \"Intel(R) KeemBay XLink PCIe Device\" in device manager, then enable them in reverse order,  and  press any key to continue OS images flashing")
            else:
                self.logger.warning("Shutdown the system, set the boot switch to normal and reboot the system and run './flashtool flashall IMAGES...' to continue OS images flashing")
                exit()
        else:
            self.logger.warning("FIP flashing failed")
            exit()


    def flash_all(self, **kwargs):
        max_threads = kwargs.get("max_threads", False)
        fip = kwargs.get("fip", None)
        if fip:
            if os.path.basename(fip) == "fip-evm.bin":
                raise click.BadParameter("For FIP flashing on EVM PCIe board, please use recover command.")
            self.recover(fip)

        if kwargs.get("wipe", False):
            self.logger.warning("Partition erasing is not enabled for Windows")

        partition_img_mapping = {
            "boot": "boot_a",
            "system": "system_a",
            "syshash": "syshash_a",
            "data": "data",
        }

        win_part_img_mapping = {
            "boot": c_uint32(0),
            "system": c_uint32(1),
            "syshash": c_uint32(2),
            "data": c_uint32(6),
        }
        win_file_name = {
            "boot": b"C:\\intel-kmb\\boot.img",
            "system": b"C:\\intel-kmb\\system.img",
            "syshash": b"C:\\intel-kmb\\syshash.img",
            "data": b"C:\\intel-kmb\\data.img",
        }

        win_image_name = {
            "boot": b"intel-kmb\\boot.img",
            "system": b"intel-kmb\\system.img",
            "syshash": b"intel-kmb\\syshash.img",
            "data": b"intel-kmb\\data.img",
        }
        secondary_mapping = {
            "boot": "boot_b",
            "system": "system_b",
            "syshash": "syshash_b",
        }

        flashAllArgs = [os.path.join(self.xlink_path,'XLinkFlashPartition.exe')]
        images = []
        for key, val in kwargs.items():
            try:
                part_name = win_part_img_mapping[key]
                shutil.copy(val, win_file_name[key])
                image = win_image_name[key]
                images.append(image)
                flashAllArgs.append(key)
            except KeyError:
                continue

        if not images:
            return

        if kwargs.get("skip_secondary", False):
            flashAllArgs.append("skip_secondary")

        if kwargs.get("skip_boot", False):
            flashAllArgs.append("skip_boot")
            
        if kwargs.get("skip_format", False):
            flashAllArgs.append("skip_format")

        if self.slot is not None :
            flashAllArgs.append("slot")
            flashAllArgs.append(self.slot)

        # Run xlinkFlashPartition
        process = subprocess.Popen(flashAllArgs)
        process.wait()
        if( process.returncode == 0):
            self.logger.info("Partitions flashed successfully")
        else:
            self.logger.warning("Partitions flashing failed")

