from subprocess import run, PIPE, STDOUT
import os
import re

cwd = os.path.dirname(__file__)

# for subprocess.run()
# bin_rel_dir = "thirdparty"
# top_dir = os.path.abspath(os.path.join(__file__, os.pardir, os.pardir, os.pardir))
# bin_dir = os.path.join(top_dir, bin_rel_dir)


def _run_cmd(cmd, **kwds):
    """call subprocess.run().
    Returns stdout with enclosing whitespaces stripped
    """
    _ = kwds.setdefault("stdout", PIPE)
    _ = kwds.setdefault("stderr", STDOUT)
    _ = kwds.setdefault("cwd", cwd)
    completed = run(cmd, **kwds)
    args, returncode, stdout = completed.args, completed.returncode, completed.stdout
    stdout = stdout.decode("utf-8").strip()
    return args, returncode, stdout


class FIPToolError(Exception):
    def __init__(self, err_str):
        self.err_str = err_str

    def __str__(self):
        return "{}".format(self.err_str)


class FIPTool:
    def __init__(self, image):
        self.image = image

    def info(self, **kwds):
        cmd = ["./fiptool", "info", "{}".format(self.image)]
        args, returncode, stdout = _run_cmd(cmd, **kwds)
        pattern = r"^ERROR:\s+(.+)"
        m = re.match(pattern, stdout)
        if m:
            err_str = m.group(1)
            raise FIPToolError(err_str)
        return "\n{}".format(stdout)
